Package: flexsurv
Type: Package
Title: Flexible Parametric Survival and Multi-State Models
Version: 2.3.2
Date: 2024-08-16
Authors@R: c(person("Christopher", "Jackson",
         email="chris.jackson@mrc-bsu.cam.ac.uk",
         role=c("aut", "cre")),
	   person("Paul", "Metcalfe",
         role=c("ctb")),
	   person("Jordan", "Amdahl",
         role=c("ctb")),
         person("Matthew T.", "Warkentin",
         role = c("ctb")),
         person("Michael", "Sweeting",
         role = c("ctb")),
         person("Kevin", "Kunzmann",
         role = c("ctb"))
	   )
Description: Flexible parametric models for time-to-event data,
    including the Royston-Parmar spline model, generalized gamma and
    generalized F distributions.  Any user-defined parametric
    distribution can be fitted, given at least an R function defining
    the probability density or hazard. There are also tools for
    fitting and predicting from fully parametric multi-state models,
    based on either cause-specific hazards or mixture models.
License: GPL (>= 2)
Depends: survival, R (>= 2.15.0)
Imports: assertthat, deSolve, generics, magrittr, mstate (>= 0.2.10),
        Matrix, muhaz, mvtnorm, numDeriv, quadprog, Rcpp (>= 0.11.5),
        rlang, rstpm2, purrr, statmod, tibble, tidyr, dplyr,
        tidyselect, ggplot2
Encoding: UTF-8
Suggests: splines2, flexsurvcure, survminer, lubridate, rmarkdown,
        colorspace, eha, knitr, msm, testthat, TH.data, broom, covr
URL: https://github.com/chjackson/flexsurv,
        http://chjackson.github.io/flexsurv/
BugReports: https://github.com/chjackson/flexsurv/issues
VignetteBuilder: knitr
LazyData: yes
LinkingTo: Rcpp
RoxygenNote: 7.3.1
NeedsCompilation: yes
Packaged: 2024-08-16 15:05:50 UTC; Chris
Author: Christopher Jackson [aut, cre],
  Paul Metcalfe [ctb],
  Jordan Amdahl [ctb],
  Matthew T. Warkentin [ctb],
  Michael Sweeting [ctb],
  Kevin Kunzmann [ctb]
Maintainer: Christopher Jackson <chris.jackson@mrc-bsu.cam.ac.uk>
Repository: CRAN
Date/Publication: 2024-08-17 05:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 10:09:45 UTC; unix
Archs: flexsurv.so.dSYM
