# Generated by roxygen2: do not edit by hand

S3method(AIC,fmsm)
S3method(AICC,flexsurvreg)
S3method(AICc,flexsurvreg)
S3method(BIC,flexsurvreg)
S3method(augment,flexsurvreg)
S3method(coef,flexsurvreg)
S3method(glance,flexsurvreg)
S3method(lines,flexsurvreg)
S3method(lines,survrtrunc)
S3method(logLik,flexsurvreg)
S3method(model.frame,flexsurvmix)
S3method(model.frame,flexsurvreg)
S3method(model.matrix,flexsurvreg)
S3method(nobs,flexsurvreg)
S3method(plot,flexsurvreg)
S3method(plot,standsurv)
S3method(plot,survrtrunc)
S3method(predict,flexsurvreg)
S3method(print,flexsurvmix)
S3method(print,flexsurvreg)
S3method(print,flexsurvrtrunc)
S3method(print,fmsm)
S3method(print,summary.flexsurvreg)
S3method(print,totlos.fs)
S3method(residuals,flexsurvreg)
S3method(simulate,flexsurvreg)
S3method(summary,flexsurvreg)
S3method(summary,flexsurvrtrunc)
S3method(tidy,flexsurvreg)
S3method(tidy,standsurv)
S3method(vcov,flexsurvreg)
export(AICC)
export(AICc)
export(Hexp)
export(Hgamma)
export(Hgenf)
export(Hgenf.orig)
export(Hgengamma)
export(Hgengamma.orig)
export(Hgompertz)
export(Hllogis)
export(Hlnorm)
export(Hsurvspline)
export(Hsurvspline0)
export(Hsurvspline1)
export(Hsurvspline2)
export(Hsurvspline3)
export(Hsurvspline4)
export(Hsurvspline5)
export(Hsurvspline6)
export(Hsurvspline7)
export(Hweibull)
export(HweibullPH)
export(ajfit)
export(ajfit_flexsurvmix)
export(ajfit_fmsm)
export(augment)
export(basis)
export(bootci.fmsm)
export(coxsnell_flexsurvreg)
export(dbasis)
export(dfss)
export(dgenf)
export(dgenf.orig)
export(dgengamma)
export(dgengamma.orig)
export(dgompertz)
export(dllogis)
export(dsurvspline)
export(dsurvspline0)
export(dsurvspline1)
export(dsurvspline2)
export(dsurvspline3)
export(dsurvspline4)
export(dsurvspline5)
export(dsurvspline6)
export(dsurvspline7)
export(dweibullPH)
export(flexsurv.dists)
export(flexsurvmix)
export(flexsurvreg)
export(flexsurvrtrunc)
export(flexsurvspline)
export(fmixmsm)
export(fmsm)
export(fss)
export(glance)
export(hexp)
export(hgamma)
export(hgenf)
export(hgenf.orig)
export(hgengamma)
export(hgengamma.orig)
export(hgompertz)
export(hllogis)
export(hlnorm)
export(hr_flexsurvreg)
export(hsurvspline)
export(hsurvspline0)
export(hsurvspline1)
export(hsurvspline2)
export(hsurvspline3)
export(hsurvspline4)
export(hsurvspline5)
export(hsurvspline6)
export(hsurvspline7)
export(hweibull)
export(hweibullPH)
export(mean_exp)
export(mean_flexsurvmix)
export(mean_gamma)
export(mean_genf)
export(mean_genf.orig)
export(mean_gengamma)
export(mean_gengamma.orig)
export(mean_gompertz)
export(mean_llogis)
export(mean_lnorm)
export(mean_survspline)
export(mean_survspline0)
export(mean_survspline1)
export(mean_survspline2)
export(mean_survspline3)
export(mean_survspline4)
export(mean_survspline5)
export(mean_survspline6)
export(mean_survspline7)
export(mean_weibull)
export(mean_weibullPH)
export(meanfinal_fmixmsm)
export(msfit.flexsurvreg)
export(normboot.flexsurvreg)
export(p_flexsurvmix)
export(pars.fmsm)
export(pdf_flexsurvmix)
export(pfinal_fmsm)
export(pgenf)
export(pgenf.orig)
export(pgengamma)
export(pgengamma.orig)
export(pgompertz)
export(pllogis)
export(pmatrix.fs)
export(pmatrix.simfs)
export(ppath_fmixmsm)
export(probs_flexsurvmix)
export(psurvspline)
export(psurvspline0)
export(psurvspline1)
export(psurvspline2)
export(psurvspline3)
export(psurvspline4)
export(psurvspline5)
export(psurvspline6)
export(psurvspline7)
export(pweibullPH)
export(qfinal_fmixmsm)
export(qgeneric)
export(qgenf)
export(qgenf.orig)
export(qgengamma)
export(qgengamma.orig)
export(qgompertz)
export(qllogis)
export(qsurvspline)
export(qsurvspline0)
export(qsurvspline1)
export(qsurvspline2)
export(qsurvspline3)
export(qsurvspline4)
export(qsurvspline5)
export(qsurvspline6)
export(qsurvspline7)
export(quantile_flexsurvmix)
export(qweibullPH)
export(rgenf)
export(rgenf.orig)
export(rgengamma)
export(rgengamma.orig)
export(rgompertz)
export(rllogis)
export(rmst_exp)
export(rmst_flexsurvmix)
export(rmst_gamma)
export(rmst_generic)
export(rmst_genf)
export(rmst_genf.orig)
export(rmst_gengamma)
export(rmst_gengamma.orig)
export(rmst_gompertz)
export(rmst_llogis)
export(rmst_lnorm)
export(rmst_survspline)
export(rmst_survspline0)
export(rmst_survspline1)
export(rmst_survspline2)
export(rmst_survspline3)
export(rmst_survspline4)
export(rmst_survspline5)
export(rmst_survspline6)
export(rmst_survspline7)
export(rmst_weibull)
export(rmst_weibullPH)
export(rsurvspline)
export(rsurvspline0)
export(rsurvspline1)
export(rsurvspline2)
export(rsurvspline3)
export(rsurvspline4)
export(rsurvspline5)
export(rsurvspline6)
export(rsurvspline7)
export(rweibullPH)
export(sim.fmsm)
export(simfinal_fmsm)
export(simfs_bytrans)
export(standsurv)
export(survrtrunc)
export(tidy)
export(totlos.fs)
export(totlos.simfs)
export(unroll.function)
import(ggplot2)
import(rlang)
import(stats)
importFrom(Matrix,nearPD)
importFrom(Rcpp,sourceCpp)
importFrom(deSolve,ode)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(generics,augment)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(magrittr,"%>%")
importFrom(mstate,msfit)
importFrom(mstate,probtrans)
importFrom(muhaz,muhaz)
importFrom(mvtnorm,rmvnorm)
importFrom(numDeriv,grad)
importFrom(numDeriv,hessian)
importFrom(purrr,map2_dbl)
importFrom(quadprog,solve.QP)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(statmod,gauss.quad)
importFrom(stats,predict)
importFrom(stats,residuals)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,survfit)
importFrom(survival,survreg)
importFrom(survival,survreg.control)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unnest)
importFrom(tidyselect,all_of)
importFrom(tidyselect,matches)
importFrom(tidyselect,num_range)
importFrom(utils,globalVariables)
useDynLib(flexsurv, .registration = TRUE)
