# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

dgenf_work <- function(x, mu, sigma, Q, P, log) {
    .Call(`_flexsurv_dgenf_work`, x, mu, sigma, Q, P, log)
}

pgenf_work <- function(q, mu, sigma, Q, P, lower_tail, give_log) {
    .Call(`_flexsurv_pgenf_work`, q, mu, sigma, Q, P, lower_tail, give_log)
}

check.genf <- function(mu, sigma, Q, P) {
    .Call(`_flexsurv_check_genf`, mu, sigma, Q, P)
}

dgengamma_work <- function(x, mu, sigma, Q, log) {
    .Call(`_flexsurv_dgengamma_work`, x, mu, sigma, Q, log)
}

pgengamma_work <- function(q, mu, sigma, Q, lower_tail, give_log) {
    .Call(`_flexsurv_pgengamma_work`, q, mu, sigma, Q, lower_tail, give_log)
}

check.gengamma <- function(mu, sigma, Q) {
    .Call(`_flexsurv_check_gengamma`, mu, sigma, Q)
}

dgompertz_work <- function(x, shape, rate, log) {
    .Call(`_flexsurv_dgompertz_work`, x, shape, rate, log)
}

pgompertz_work <- function(q, shape, rate, lower_tail, give_log) {
    .Call(`_flexsurv_pgompertz_work`, q, shape, rate, lower_tail, give_log)
}

check.gompertz <- function(shape, rate) {
    .Call(`_flexsurv_check_gompertz`, shape, rate)
}

dllogis_work <- function(x, shape, scale, log) {
    .Call(`_flexsurv_dllogis_work`, x, shape, scale, log)
}

pllogis_work <- function(q, shape, scale, lower_tail, give_log) {
    .Call(`_flexsurv_pllogis_work`, q, shape, scale, lower_tail, give_log)
}

check.llogis <- function(shape, scale) {
    .Call(`_flexsurv_check_llogis`, shape, scale)
}

exph <- function(y) {
    .Call(`_flexsurv_exph`, y)
}

dexph <- function(y) {
    .Call(`_flexsurv_dexph`, y)
}

basis_vector <- function(knots, x) {
    .Call(`_flexsurv_basis_vector`, knots, x)
}

basis_matrix <- function(knots, x) {
    .Call(`_flexsurv_basis_matrix`, knots, x)
}

dbasis_vector <- function(knots, x) {
    .Call(`_flexsurv_dbasis_vector`, knots, x)
}

dbasis_matrix <- function(knots, x) {
    .Call(`_flexsurv_dbasis_matrix`, knots, x)
}

