% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/standsurv.R
\name{tidy.standsurv}
\alias{tidy.standsurv}
\title{Tidy a standsurv object.}
\usage{
\method{tidy}{standsurv}(x, ...)
}
\arguments{
\item{x}{A standsurv object.}

\item{...}{Not currently used.}
}
\value{
Returns additional tidy data.frames (tibbles)
stored as attributes named standpred_at and standpred_contrast.
}
\description{
This function is used internally by \code{standsurv} and tidy
data.frames are automatically returned by the function.
}
